@@ -3,7 +3,7 @@
 __title__ = 'unicaps'
 __description__ = 'Python CAPTCHA solving for Humans.'
 __url__ = 'https://github.com/sergey-scat/unicaps'
-__version__ = '0.5.1'
-__author__ = 'Sergey Totmyanin'
+__version__ = '1.0.0'
+__author__ = 'Sergey Scat'
 __license__ = 'Apache 2.0'
-__copyright__ = 'Copyright 2020-2021 Sergey Totmyanin'
+__copyright__ = 'Copyright 2020-2022 Sergey Scat'
