@@ -6,9 +6,12 @@ FunCaptcha
 from dataclasses import dataclass
 from typing import Optional
 
+from enforce_typing import enforce_types  # type: ignore
+
 from .base import BaseCaptcha, BaseCaptchaSolution
 
 
+@enforce_types
 @dataclass
 class FunCaptcha(BaseCaptcha):
     """ FunCaptcha """
@@ -17,14 +20,10 @@ class FunCaptcha(BaseCaptcha):
     page_url: str
     service_url: Optional[str] = None
     no_js: Optional[bool] = None
-
-    def __post_init__(self):
-        assert isinstance(self.public_key, str)
-        assert isinstance(self.page_url, str)
-        assert self.service_url is None or isinstance(self.service_url, str)
-        assert self.no_js is None or isinstance(self.no_js, bool)
+    blob: Optional[str] = None
 
 
+@enforce_types
 @dataclass
 class FunCaptchaSolution(BaseCaptchaSolution):
     """ FunCaptcha solution """
