@@ -4,22 +4,25 @@ hCaptcha
 """
 
 from dataclasses import dataclass
+from typing import Optional
+
+from enforce_typing import enforce_types  # type: ignore
 
 from .base import BaseCaptcha, BaseCaptchaSolution
 
 
+@enforce_types
 @dataclass
 class HCaptcha(BaseCaptcha):
     """ hCaptcha """
 
     site_key: str
     page_url: str
-
-    def __post_init__(self):
-        assert isinstance(self.site_key, str)
-        assert isinstance(self.page_url, str)
+    is_invisible: bool = False
+    api_domain: Optional[str] = None
 
 
+@enforce_types
 @dataclass
 class HCaptchaSolution(BaseCaptchaSolution):
     """ hCaptcha solution """
