@@ -6,27 +6,25 @@ Google reCAPTCHA v3
 from dataclasses import dataclass
 from typing import Optional
 
+from enforce_typing import enforce_types  # type: ignore
+
 from .base import BaseCaptcha, BaseCaptchaSolution
 
 
+@enforce_types
 @dataclass
 class RecaptchaV3(BaseCaptcha):
     """ Google reCAPTCHA v3 """
 
     site_key: str
     page_url: str
-    is_enterprise: Optional[bool] = False
+    is_enterprise: bool = False
     action: Optional[str] = None
     min_score: Optional[float] = None
-
-    def __post_init__(self):
-        assert isinstance(self.site_key, str)
-        assert isinstance(self.page_url, str)
-        assert isinstance(self.is_enterprise, bool)
-        assert self.action is None or isinstance(self.action, str)
-        assert self.min_score is None or isinstance(self.min_score, float)
+    api_domain: Optional[str] = None
 
 
+@enforce_types
 @dataclass
 class RecaptchaV3Solution(BaseCaptchaSolution):
     """ Google reCAPTCHA v3 solution """
